require 'cgi'
require 'openssl'
OpenSSL::SSL.send(:remove_const, :VERIFY_PEER)
OpenSSL::SSL::VERIFY_PEER = OpenSSL::SSL::VERIFY_NONE

module OsmHelper
  def osm_node_url(osmid)
    osmid.nil? ? nil: "http://openstreetmap.org/node/#{osmid}"
  end
  def osm_relation_url(osmid); "http://openstreetmap.org/relation/#{osmid}" end
  def waymarkedtrails_relation_url(osmid); "http://hiking.waymarkedtrails.org/cs/relation/#{osmid}"   end
  def waymarkedtrails_position_url(coords); "http://hiking.waymarkedtrails.org/#?map=18!#{coords[0]}!#{coords[1]}"   end
  def josm_relation_load_url(osmid);   "http://localhost:8111/import?url=http://api.openstreetmap.org/api/0.6/relation/#{osmidize(osmid)}/full" end
  
  def josm_zoom_to_points(*points)
    josm_zoom_to_points_with_zoom(1,*points)
  end
  def josm_zoom_to_points_with_zoom(zoom, *points)
    "http://localhost:8111/load_and_zoom?" + query_encapsulating_points_with_zoom(zoom, *points)
  end
  def josm_select_relation_and_zoom_to_points(relationid,*points)
    josm_zoom_to_points(*points)+"&select=r#{relationid}"
  end
  def josm_add_node(point, tags={})
    tags_part = escape_tag_part_for_query(tags)
    tags_part = '&' + {:addtags => tags_part}.to_query unless tags_part.empty?
    return "http://localhost:8111/add_node?lon=#{point[1]}&lat=#{point[0]}#{tags_part}"
  end
  def query_encapsulating_points(*points)
    query_encapsulating_points_with_zoom(1,*points)
  end
  def query_encapsulating_points_with_zoom(zoom, *points)
    y_coords = points.collect{|point| point.slice(0)}
    x_coords = points.collect{|point| point.slice(1)}
    y_diff = y_coords.max - y_coords.min
    x_diff = x_coords.max - x_coords.min
    min_diff = 0.0001
    if zoom==1
      diff = [x_diff,y_diff,min_diff].max
    else
      diff = [x_diff,y_diff,min_diff].max*(zoom-1)
    end
    right = x_coords.max
    "left=#{x_coords.min-diff}&right=#{x_coords.max+diff}&bottom=#{y_coords.min-diff}&top=#{y_coords.max+diff}"
  end
  def select_node_and_add_tags(lat,lon,nodeid,tags={})
    josm_zoom_to_points([lat,lon])+"&select=n#{nodeid}"+"&addtags="+escape_tag_part_for_query(tags)
  end
  def josm_load_objects(*objects)
    objstring = objects.collect do |obj|
      case obj
        when OsmGuidepost then 'n'
        when OsmTrail then 'r'
        else raise "Objekt třídy #{obj.class} nemohu nahrát z josm."
      end+obj.osmid.to_s
    end.join(',')
    return "http://localhost:8111/load_object?new_layer=false&relation_members=true&objects="+objstring
  end
  # vloží guidepost do trailu
  def josm_add_osm_guideposts_to_osm_trail(osm_trail, *osm_guideposts) 
    raise unless osm_trail.is_a? OsmTrail
    data = data_for_osm_trail(osm_trail.osmid)
    osm_guideposts.each do |osm_guidepost|
      data.sub!(/<tag/,"<member type='node' ref='#{osm_guidepost.osmid}' role='guidepost' />\n<tag")  
    end
    data.sub!(/<relation/,"<relation action='modify'")
    return  "http://localhost:8111/load_data?#{data.to_query('data')}"
  end
  # vyjme osm_guidepost z osm_trailu
  def josm_delete_osm_guidepost_from_osm_trail(osm_trail, osm_guidepost)
    data = data_for_osm_trail(osm_trail.osmid)
    if data.nil?
      puts "OSM Trasa s osmid=#{osm_trail.osmid} zmizela, asi bude třeba přeimportovat"
      return nil
    end
    
    result = data.sub!(/<member type=.node. +ref=.#{osm_guidepost.osmid}. +role=.\w*.\s*\/?>/,'')
    if result.nil?
      puts "OSM rozcestnik og:#{osm_guidepost.osmid} na ceste ot:#{osm_trail.osmid} neni, nutno preimportovat"
      return nil
    end
    
    return  "http://localhost:8111/load_data?#{data.to_query('data')}"
  end
  def data_for_osm_trail(osmid)
    return open("https://www.openstreetmap.org/api/0.6/relation/#{osmid}"){|io| io.read}
  rescue OpenURI::HTTPError
    return nil
  end
  # vytvori duplikat osm_trasy od zacatku az po urcity sign
  def josm_duplicate_part_of_trail_up_to_sign_url(osm_sign)  
  end
  
  private
  # připrav tagy do formátu pro url query
  def escape_tag_part_for_query(tags)
    CGI.escape(tags.keys.collect{ |key| "#{key}=#{tags[key]}"}.join('|'))
  end
  def data_placeholder
    <<-EOS
<?xml version='1.0' encoding='UTF-8'?>
<osm version='0.6' generator='JOSM'>
  <relation id='2667978' action='modify' timestamp='2017-06-15T02:33:02Z' uid='10353' user='gorn' version='6' changeset='49545899'>
    <member type='way' ref='145760297' role='' />
    <member type='way' ref='142802035' role='' />
    <member type='way' ref='198659463' role='' />
    <member type='way' ref='142954097' role='' />
    <member type='way' ref='198659471' role='' />
    <member type='way' ref='114461513' role='' />
    <member type='way' ref='219615192' role='' />
    <member type='way' ref='73485315' role='' />
    <member type='way' ref='73485308' role='' />
    <member type='way' ref='231912570' role='' />
    <member type='way' ref='140780817' role='' />
    <member type='way' ref='140780815' role='' />
    <member type='way' ref='140780818' role='' />
    <member type='way' ref='140780816' role='' />
    <member type='way' ref='140780819' role='' />
    <member type='way' ref='435644117' role='' />
    <member type='way' ref='435644116' role='' />
    <member type='way' ref='271823493' role='' />
    <tag k='complete' v='no' />
    <tag k='destinations' v='Starý Plzenec - Sedlec - Pod Sedleckou skálou - Svidná' />
    <tag k='kct_green' v='major' />
    <tag k='network' v='rwn' />
    <tag k='operator' v='cz:KČT' />
    <tag k='osmc:symbol' v='green:white:green_bar' />
    <tag k='ref' v='3660' />
    <tag k='route' v='hiking' />
    <tag k='type' v='route' />
  </relation>
</osm>
    EOS
  end
  # pokud je parametr OsmTrail tak ho predelej na osmid, pokud je to osmid, nech ho tak
  def osmidize(unknown)
    unknown = unknown.osmid if unknown.is_a? OsmTrail
    return unknown
  end
  
=begin
  def josm_export_segment(point1,point2)
    "http://localhost:8111/load_data?new_layer=yes&layer_name=terrain_notes" + '&' +
    "#{data.to_query('data')}" 
  end
  def data
    <<-EOS
<?xml version='1.0' encoding='UTF-8'?>
      <gpx version="1.1" xmlns="http://www.topografix.com/GPX/1/1"
        xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" 
        xsi:schemaLocation="http://www.topografix.com/GPX/1/1http://www.topografix.com/GPX/1/1/gpx.xsd">
      <metadata>
        <desc>Errors in znacky</desc>
        <bounds minlat="49.7431575" minlon="13.7263094" maxlat="49.7435786" maxlon="13.7355099"/>
      </metadata>
      <wpt lat="49.74357860413543" lon="13.735509914636793">
        <name>kam dále vede zelená</name>
      </wpt>
      <wpt lat="49.74315746395922" lon="13.726309365229676">
        <name>kam dále vede zelená</name>
      </wpt>
      <trk>
        <name>chybějící segment zelené</name>
        <trkseg>
          <trkpt lat="49.74357413371818" lon="13.735395048831124"/>
          <trkpt lat="49.74316515996139" lon="13.726511997767318"/>
        </trkseg>
      </trk>
      </gpx>
    EOS
  end
=end
end
